Module: Roda::RodaPlugins::Kabk::RequestMethods
- Defined in:
- lib/roda/plugins/kabk.rb
Overview
Request methods added to the Roda request instance.
Instance Method Summary collapse
-
#kabk ⇒ RodaPlugins::Kabk::Helpers
Returns the Kabk helper object bound to the current request and plugin options.
Instance Method Details
#kabk ⇒ RodaPlugins::Kabk::Helpers
Returns the Kabk helper object bound to the current request and plugin options.
45 46 47 48 |
# File 'lib/roda/plugins/kabk.rb', line 45 def kabk = roda_class.opts[:kabk] || (scope.opts[:kabk] if respond_to?(:scope)) || {} @kabk_helpers ||= ::RodaPlugins::Kabk::Helpers.new(self, ) end |